@@ -515,7 +515,9 @@ public:
 			server.detach(bsWrapper);
 		}
 		// similar effect like "delete this;"		
+		Arts::ByteSoundProducerV2_base* x = _copy();
 		bsWrapper = ByteSoundProducerV2::null();
+		x->_release();
 	}
 
 	int write(const mcopbyte *data, int size)
