@@ -46,7 +46,7 @@
 /*
  * NOTE:
  *
- * To truely support non-blocking I/O, there is some stuff missing. Namely,
+ * To truly support non-blocking I/O, there is some stuff missing. Namely,
  * select should be trapped and redirected, as well as the means for making
  * a stream non-blocking and so on. Maybe poll, too.
  *
@@ -85,7 +85,7 @@ typedef unsigned long int ioctl_request_t;
 static int mmapemu = 0;
 static count_info mmapemu_ocount;
 static char *mmapemu_obuffer = 0;
-static int mmapemu_osize;
+static int mmapemu_osize = 0;
 
 /*
  * original C library functions
@@ -164,7 +164,13 @@ static void artsdspdebug(const char *fmt,...)
 
 static void mmapemu_flush()
 {
-  int space = arts_stream_get(stream, ARTS_P_BUFFER_SPACE);
+  int space;
+
+  if (mmapemu_osize == 0) {
+    return;
+  }
+
+  space = arts_stream_get(stream, ARTS_P_BUFFER_SPACE);
   artsdspdebug("space = %d\n",space);
   while(space >= 4096)
   {
@@ -587,7 +593,7 @@ void *mmap(void  *start,  size_t length, int prot, int flags,
                                                           start, length, prot);
     artsdspdebug ("      flags = %d, fd = %d, offset = %d\n",flags, fd,offset);
 
-	if(mmapemu)
+	if(mmapemu && length > 0)
 	{
       mmapemu_osize = length;
       mmapemu_obuffer = malloc(length);
