@@ -45,7 +45,7 @@ public:
    a typedef in between, which makes it magically work.
    We could also use an explicit instantiation, but this is not allowed
    on all C++ compilers in this scope.  Note also, that we don't need
-   to replace _all_ occurences of list<xxx*> below, only the two in the
+   to replace _all_ occurrences of list<xxx*> below, only the two in the
    member declaration.  What a mess.  */
 typedef list<AudioManagerClient_impl *> L_AMC;
 typedef list<AudioManagerAssignable *> L_AMA;
