@@ -47,8 +47,10 @@
 
 
 /* --- variables --- */
-volatile guint64 gsl_externvar_tick_stamp = 0;
-static guint     global_tick_stamp_leaps = 0;
+volatile guint64     gsl_externvar_tick_stamp = 0;
+static guint64	     tick_stamp_system_time = 0;
+static guint         global_tick_stamp_leaps = 0;
+static GslDebugFlags gsl_debug_flags = 0;
 
 
 /* --- memory allocation --- */
@@ -161,8 +163,29 @@ gsl_free_memblock (gsize    block_size,
 void
 gsl_alloc_report (void)
 {
-  g_message ("Gsl-Memory: %lu bytes currently used",
-	     memory_allocated);
+  guint cell, cached = 0;
+
+  GSL_SPIN_LOCK (&global_memory);
+  for (cell = 0; cell < SIMPLE_CACHE_SIZE; cell++)
+    {
+      GTrashStack *trash = simple_cache[cell];
+      guint memsize, n = 0;
+
+      while (trash)
+	{
+	  n++;
+	  trash = trash->next;
+	}
+
+      if (n)
+	{
+	  memsize = (cell + 1) << 3;
+	  g_message ("cell %4u): %u bytes in %u nodes", memsize, memsize * n, n);
+	  cached += memsize * n;
+	}
+    }
+  g_message ("%lu bytes allocated from system, %u bytes unused in cache", memory_allocated, cached);
+  GSL_SPIN_UNLOCK (&global_memory);
 }
 
 gpointer
@@ -213,8 +236,7 @@ gsl_ring_prepend_i (GslRing  *head,
     {
       ring->prev = head->prev;
       ring->next = head;
-      if (head->prev)
-	head->prev->next = ring;
+      head->prev->next = ring;
       head->prev = ring;
     }
   return ring;
@@ -409,6 +431,44 @@ gsl_ring_pop_tail (GslRing **head_p)
   return data;
 }
 
+GslRing*
+gsl_ring_insert_sorted (GslRing	    *head,
+			gpointer     data,
+			GCompareFunc func)
+{
+  gint cmp;
+
+  g_return_val_if_fail (func != NULL, head);
+
+  if (!head)
+    return gsl_ring_prepend (head, data);
+
+  /* typedef gint (*GCompareFunc) (gconstpointer a,
+   *                               gconstpointer b);
+   */
+  cmp = func (data, head->data);
+
+  if (cmp >= 0)	/* insert after head */
+    {
+      GslRing *tmp, *tail = head->prev;
+      
+      /* make appending an O(1) operation */
+      if (head == tail || func (data, tail->data) >= 0)
+	return gsl_ring_append (head, data);
+
+      /* walk forward while data >= tmp (skipping equal nodes) */
+      for (tmp = head->next; tmp != tail; tmp = tmp->next)
+	if (func (data, tmp->data) < 0)
+	  break;
+
+      /* insert before sibling which is greater than data */
+      gsl_ring_prepend (tmp, data);	/* keep current head */
+      return head;
+    }
+  else /* cmp < 0 */
+    return gsl_ring_prepend (head, data);
+}
+
 
 /* --- GslThread --- */
 typedef struct
@@ -418,12 +478,15 @@ typedef struct
   gint		wpipe[2];
   volatile gint abort;
   guint64       awake_stamp;
+  GslDebugFlags auxlog_reporter;
+  const gchar  *auxlog_section;
 } ThreadData;
 static GslMutex    global_thread = { 0, };
 static GslRing    *global_thread_list = NULL;
 static GslCond     global_thread_cond = { 0, };
 static GslRing    *awake_tdata_list = NULL;
 static ThreadData *main_thread_tdata = NULL;
+static GslThread  *main_thread = NULL;
 
 static inline ThreadData*
 thread_data_from_gsl_thread (GslThread *thread)
@@ -479,6 +542,8 @@ create_tdata (void)
   tdata->wpipe[0] = -1;
   tdata->wpipe[1] = -1;
   tdata->abort = FALSE;
+  tdata->auxlog_reporter = 0;
+  tdata->auxlog_section = NULL;
   error = pipe (tdata->wpipe);
   if (error == 0)
     {
@@ -508,7 +573,6 @@ GslThread*
 gsl_thread_new (GslThreadFunc func,
 		gpointer      user_data)
 {
-  const gboolean joinable = TRUE;
   gpointer gthread = NULL;
   ThreadData *tdata;
   GError *gerror = NULL;
@@ -519,6 +583,12 @@ gsl_thread_new (GslThreadFunc func,
 
   if (tdata)
     {
+      const gboolean joinable = FALSE;
+
+      /* don't dare setting joinable to TRUE, that prevents the thread's
+       * resources from being freed, since we don't offer pthread_join().
+       * so we'd just rn out of stack at some point.
+       */
       tdata->func = func;
       tdata->data = user_data;
       gthread = g_thread_create_full (thread_wrapper, tdata, 0, joinable, FALSE,
@@ -558,6 +628,12 @@ gsl_thread_self (void)
   return gthread;
 }
 
+GslThread*
+gsl_thread_main (void)
+{
+  return main_thread;
+}
+
 guint
 gsl_threads_get_count (void)
 {
@@ -617,6 +693,7 @@ gsl_thread_abort (GslThread *thread)
   ThreadData *tdata;
   
   g_return_if_fail (thread != NULL);
+  g_return_if_fail (thread != main_thread);
   
   GSL_SYNC_LOCK (&global_thread);
   g_assert (gsl_ring_find (global_thread_list, thread));
@@ -647,7 +724,8 @@ gsl_thread_queue_abort (GslThread *thread)
   ThreadData *tdata;
   
   g_return_if_fail (thread != NULL);
-
+  g_return_if_fail (thread != main_thread);
+  
   GSL_SYNC_LOCK (&global_thread);
   g_assert (gsl_ring_find (global_thread_list, thread));
   GSL_SYNC_UNLOCK (&global_thread);
@@ -685,7 +763,8 @@ gsl_thread_aborted (void)
  * @returns:  %TRUE if the thread should continue execution
  * Sleep for the amount of time given. This function may get interrupted
  * by wakeup or abort requests, it returns whether the thread is supposed
- * to continue execution after waking up.
+ * to continue execution after waking up. This function also processes
+ * remaining data from the thread's poll fd.
  */
 gboolean
 gsl_thread_sleep (glong max_msec)
@@ -695,14 +774,14 @@ gsl_thread_sleep (glong max_msec)
   gint r, aborted;
 
   pfd.fd = tdata->wpipe[0];
-  pfd.events = GSL_POLLIN;
+  pfd.events = G_IO_IN;
   pfd.revents = 0;
 
   r = poll (&pfd, 1, max_msec);
 
   if (r < 0 && errno != EINTR)
     g_message (G_STRLOC ": poll() error: %s\n", g_strerror (errno));
-  else if (pfd.revents & GSL_POLLIN)
+  else if (pfd.revents & G_IO_IN)
     {
       guint8 data[64];
 
@@ -718,13 +797,20 @@ gsl_thread_sleep (glong max_msec)
   return !aborted;
 }
 
+/**
+ * gsl_thread_awake_after
+ * RETURNS: GPollFD for the current thread
+ * Get the GPollfd for the current thread which is used
+ * to signal thread wakeups (e.g. due to
+ * gsl_thread_abort() or gsl_thread_wakeup()).
+ */
 void
-gsl_thread_get_pollfd (GslPollFD *pfd)
+gsl_thread_get_pollfd (GPollFD *pfd)
 {
   ThreadData *tdata = thread_data_from_gsl_thread (gsl_thread_self ());
 
   pfd->fd = tdata->wpipe[0];
-  pfd->events = GSL_POLLIN;
+  pfd->events = G_IO_IN;
   pfd->revents = 0;
 }
 
@@ -777,6 +863,7 @@ gsl_thread_awake_before (guint64 tick_stamp)
 /**
  * gsl_tick_stamp
  * @RETURNS: GSL's execution tick stamp as unsigned 64bit integer
+ *
  * Retrive the GSL global tick stamp.
  * GSL increments its global tick stamp at certain intervals,
  * by specific amounts (refer to gsl_engine_init() for further
@@ -789,6 +876,7 @@ gsl_thread_awake_before (guint64 tick_stamp)
  * GslProcessFunc() functions) may use the macro %GSL_TICK_STAMP
  * to retrive the current tick in a faster manner (not involving
  * mutex locking). See also gsl_module_tick_stamp().
+ * This function is MT-safe and may be called from any thread.
  */
 guint64
 gsl_tick_stamp (void)
@@ -810,18 +898,68 @@ _gsl_tick_stamp_set_leap (guint ticks)
   GSL_SYNC_UNLOCK (&global_thread);
 }
 
+/**
+ * gsl_time_system
+ * @RETURNS: Current system time in micro seconds
+ *
+ * Get the current system time in micro seconds.
+ * Subsequent calls to this function do not necessarily
+ * return growing values. In fact, a second call may return
+ * a value smaller than the first call under certainsystem
+ * conditions.
+ * This function is MT-safe and may be called from any thread.
+ */
+guint64
+gsl_time_system (void)
+{
+  struct timeval tv;
+  guint64 csys_time;
+  gint error;
+
+  error = gettimeofday (&tv, NULL);
+  if (error)
+    g_error ("gettimeofday() failed: %s", g_strerror (errno));
+  csys_time = tv.tv_sec;
+  csys_time = csys_time * 1000000 + tv.tv_usec;
+
+  return csys_time;
+}
+
+/**
+ * gsl_tick_stamp_last
+ * @RETURNS: Current tick stamp and system time in micro seconds
+ *
+ * Get the system time of the last GSL global tick stamp update.
+ * This function is MT-safe and may be called from any thread.
+ */
+GslTickStampUpdate
+gsl_tick_stamp_last (void)
+{
+  GslTickStampUpdate ustamp;
+
+  GSL_SYNC_LOCK (&global_thread);
+  ustamp.tick_stamp = gsl_externvar_tick_stamp;
+  ustamp.system_time = tick_stamp_system_time;
+  GSL_SYNC_UNLOCK (&global_thread);
+
+  return ustamp;
+}
+
 void
 _gsl_tick_stamp_inc (void)
 {
   volatile guint64 newstamp;
   GslRing *ring;
+  guint64 systime;
 
   g_return_if_fail (global_tick_stamp_leaps > 0);
 
+  systime = gsl_time_system ();
   newstamp = gsl_externvar_tick_stamp + global_tick_stamp_leaps;
 
   GSL_SYNC_LOCK (&global_thread);
   gsl_externvar_tick_stamp = newstamp;
+  tick_stamp_system_time = systime;
   for (ring = awake_tdata_list; ring; )
     {
       ThreadData *tdata = ring->data;
@@ -874,7 +1012,7 @@ default_mutex_lock (GslMutex *mutex)
       /* on uni processor systems, there's no point in busy spinning */
       do
 	{
-#ifdef _POSIX_PRIORITY_SCHEDULING
+#if defined(_POSIX_PRIORITY_SCHEDULING)
 	  sched_yield ();
 #endif
 	  if (g_mutex_trylock (mutex->mutex_pointer))
@@ -1089,65 +1227,220 @@ gsl_strerror (GslErrorType error)
     {
     case GSL_ERROR_NONE:		return "Everything went well";
     case GSL_ERROR_INTERNAL:		return "Internal error (please report)";
-    case GSL_ERROR_LAST:
     case GSL_ERROR_UNKNOWN:		return "Unknown error";
     case GSL_ERROR_IO:			return "I/O error";
     case GSL_ERROR_PERMS:		return "Insufficient permission";
-    case GSL_ERROR_NOT_FOUND:		return "Not found";
+    case GSL_ERROR_BUSY:		return "Resource currently busy";
+    case GSL_ERROR_EXISTS:		return "Resource exists already";
+    case GSL_ERROR_TEMP:		return "Temporary error";
+    case GSL_ERROR_EOF:			return "File empty or premature EOF";
+    case GSL_ERROR_NOT_FOUND:		return "Resource not found";
     case GSL_ERROR_OPEN_FAILED:		return "Open failed";
     case GSL_ERROR_SEEK_FAILED:		return "Seek failed";
     case GSL_ERROR_READ_FAILED:		return "Read failed";
     case GSL_ERROR_WRITE_FAILED:	return "Write failed";
-    case GSL_ERROR_EOF:			return "File empty or premature EOF";
     case GSL_ERROR_FORMAT_INVALID:	return "Invalid format";
     case GSL_ERROR_FORMAT_UNKNOWN:	return "Unknown format";
-    case GSL_ERROR_DATA_CORRUPT:        return "data corrupt";
-    case GSL_ERROR_CONTENT_GLITCH:      return "data glitch (junk) detected";
+    case GSL_ERROR_DATA_CORRUPT:        return "Data corrupt";
+    case GSL_ERROR_CONTENT_GLITCH:      return "Data glitch (junk) detected";
+    case GSL_ERROR_NO_RESOURCE:		return "Out of memory, disk space or similar resource";
     case GSL_ERROR_CODEC_FAILURE:	return "CODEC failure";
     default:				return NULL;
     }
 }
 
+static const GDebugKey gsl_static_debug_keys[] = {
+  { "notify",         GSL_MSG_NOTIFY },
+  { "dcache",         GSL_MSG_DATA_CACHE },
+  { "dhandle",        GSL_MSG_DATA_HANDLE },
+  { "loader",         GSL_MSG_LOADER },
+  { "osc",	      GSL_MSG_OSC },
+  { "engine",         GSL_MSG_ENGINE },
+  { "jobs",           GSL_MSG_JOBS },
+  { "fjobs",          GSL_MSG_FJOBS },
+  { "sched",          GSL_MSG_SCHED },
+  { "master",         GSL_MSG_MASTER },
+  { "slave",          GSL_MSG_SLAVE },
+};
+
+static const gchar*
+reporter_name (GslDebugFlags reporter)
+{
+  switch (reporter)
+    {
+    case GSL_MSG_NOTIFY:	return "Notify";
+    case GSL_MSG_DATA_CACHE:	return "DataCache";
+    case GSL_MSG_DATA_HANDLE:	return "DataHandle";
+    case GSL_MSG_LOADER:	return "Loader";
+    case GSL_MSG_OSC:		return "Oscillator";
+    case GSL_MSG_ENGINE:	return "Engine";	/* Engine */
+    case GSL_MSG_JOBS:		return "Jobs";		/* Engine */
+    case GSL_MSG_FJOBS:		return "FlowJobs";	/* Engine */
+    case GSL_MSG_SCHED:		return "Sched";		/* Engine */
+    case GSL_MSG_MASTER:	return "Master";	/* Engine */
+    case GSL_MSG_SLAVE:		return "Slave";		/* Engine */
+    default:			return "Custom";
+    }
+}
+
+const GDebugKey *gsl_debug_keys = gsl_static_debug_keys;
+const guint      gsl_n_debug_keys = G_N_ELEMENTS (gsl_static_debug_keys);
+
 void
-gsl_message_send (const gchar *reporter,
-		  GslErrorType error,
-		  const gchar *messagef,
+gsl_message_send (GslDebugFlags reporter,
+		  const gchar  *section,
+		  GslErrorType  error,
+		  const gchar  *messagef,
 		  ...)
 {
   struct {
-    gchar        reporter[1024];
-    GslErrorType error;
-    const gchar	*error_str;
-    gchar	 message[1024];
+    GslDebugFlags reporter;
+    gchar         reporter_name[64];
+    gchar         section[64];	/* auxillary information about reporter code portion */
+    GslErrorType  error;
+    const gchar	 *error_str;	/* gsl_strerror() of error */
+    gchar	  message[1024];
   } tmsg, *msg = &tmsg;
-  gchar *tmp;
+  gchar *string;
   va_list args;
     
-  g_return_if_fail (reporter != NULL);
   g_return_if_fail (messagef != NULL);
 
-  strncpy (msg->reporter, reporter, 1023);
-  msg->reporter[1023] = 0;
-
+  /* create message */
+  memset (msg, 0, sizeof (*msg));
+  msg->reporter = reporter;
+  strncpy (msg->reporter_name, reporter_name (msg->reporter), 63);
+  if (section)
+    strncpy (msg->section, section, 63);
   msg->error = error;
   msg->error_str = error ? gsl_strerror (msg->error) : NULL;
 
   /* vsnprintf() replacement */
   va_start (args, messagef);
-  tmp = g_strdup_vprintf (messagef, args);
+  string = g_strdup_vprintf (messagef, args);
   va_end (args);
-  strncpy (msg->message, tmp, 1023);
-  msg->message[1023] = 0;
-  g_free (tmp);
+  strncpy (msg->message, string, 1023);
+  g_free (string);
 
-  /* always puke the message to stderr */
-  if (msg->error_str)
-    g_printerr ("GSL **: %s: %s: %s\n", msg->reporter, msg->message, msg->error_str);
-  else
-    g_printerr ("GSL **: %s: %s\n", msg->reporter, msg->message);
+  /* in current lack of a decent message queue, puke the message to stderr */
+  g_printerr ("GSL-%s%s%s: %s%s%s\n",
+	      msg->reporter_name,
+	      msg->section ? ":" : "",
+	      msg->section ? msg->section : "",
+	      msg->message,
+	      msg->error_str ? ": " : "",
+	      msg->error_str ? msg->error_str : "");
+}
 
-  /* in current lack of a decent message queue, do nothing here */
-  ;
+void
+gsl_debug_enable (GslDebugFlags dbg_flags)
+{
+  gsl_debug_flags |= dbg_flags;
+}
+
+void
+gsl_debug_disable (GslDebugFlags dbg_flags)
+{
+  gsl_debug_flags &= dbg_flags;
+}
+
+gboolean
+gsl_debug_check (GslDebugFlags dbg_flags)
+{
+  return (gsl_debug_flags & dbg_flags) != 0;
+}
+
+void
+gsl_debug (GslDebugFlags reporter,
+	   const gchar  *section,
+	   const gchar  *format,
+	   ...)
+{
+  g_return_if_fail (format != NULL);
+
+  if (reporter & gsl_debug_flags)
+    {
+      va_list args;
+      gchar *string;
+
+      va_start (args, format);
+      string = g_strdup_vprintf (format, args);
+      va_end (args);
+      g_printerr ("DEBUG:GSL-%s%s%s: %s\n",
+		  reporter_name (reporter),
+		  section ? ":" : "",
+		  section ? section : "",
+		  string);
+      g_free (string);
+    }
+}
+
+void
+gsl_auxlog_push (GslDebugFlags reporter,
+		 const gchar  *section)
+{
+  ThreadData *tdata = thread_data_from_gsl_thread (gsl_thread_self ());
+
+  if (tdata)
+    {
+      tdata->auxlog_reporter = reporter;
+      tdata->auxlog_section = section;
+    }
+}
+
+void
+gsl_auxlog_debug (const gchar *format,
+		  ...)
+{
+  ThreadData *tdata = thread_data_from_gsl_thread (gsl_thread_self ());
+  GslDebugFlags reporter = GSL_MSG_NOTIFY;
+  const gchar *section = NULL;
+  va_list args;
+  gchar *string;
+
+  if (tdata)
+    {
+      reporter = tdata->auxlog_reporter;
+      section = tdata->auxlog_section;
+      tdata->auxlog_reporter = 0;
+      tdata->auxlog_section = NULL;
+    }
+
+  g_return_if_fail (format != NULL);
+
+  va_start (args, format);
+  string = g_strdup_vprintf (format, args);
+  va_end (args);
+  gsl_debug (reporter, section, "%s", string);
+  g_free (string);
+}
+
+void
+gsl_auxlog_message (GslErrorType error,
+		    const gchar *format,
+		    ...)
+{
+  ThreadData *tdata = thread_data_from_gsl_thread (gsl_thread_self ());
+  GslDebugFlags reporter = GSL_MSG_NOTIFY;
+  const gchar *section = NULL;
+  va_list args;
+  gchar *string;
+
+  if (tdata)
+    {
+      reporter = tdata->auxlog_reporter;
+      section = tdata->auxlog_section;
+      tdata->auxlog_reporter = 0;
+      tdata->auxlog_section = NULL;
+    }
+
+  g_return_if_fail (format != NULL);
+
+  va_start (args, format);
+  string = g_strdup_vprintf (format, args);
+  va_end (args);
+  gsl_message_send (reporter, section, error, "%s", string);
+  g_free (string);
 }
 
 
@@ -1221,16 +1514,39 @@ gsl_check_file (const gchar *file_name,
   return GSL_ERROR_NONE;
   
  have_errno:
-  switch (errno)
+  return gsl_error_from_errno (errno, GSL_ERROR_OPEN_FAILED);
+}
+
+GslErrorType
+gsl_error_from_errno (gint         sys_errno,
+		      GslErrorType fallback)
+{
+  switch (sys_errno)
     {
     case ELOOP:
     case ENAMETOOLONG:
-    case ENOENT:	return GSL_ERROR_NOT_FOUND;
+    case ENOTDIR:
+    case ENOENT:        return GSL_ERROR_NOT_FOUND;
     case EROFS:
     case EPERM:
-    case EACCES:	return GSL_ERROR_PERMS;
-    case EIO:		return GSL_ERROR_IO;
-    default:		return GSL_ERROR_OPEN_FAILED;
+    case EACCES:        return GSL_ERROR_PERMS;
+    case ENOMEM:
+    case ENOSPC:
+    case EFBIG:
+    case ENFILE:
+    case EMFILE:	return GSL_ERROR_NO_RESOURCE;
+    case EISDIR:
+    case ESPIPE:
+    case EIO:           return GSL_ERROR_IO;
+    case EEXIST:        return GSL_ERROR_EXISTS;
+    case ETXTBSY:
+    case EBUSY:         return GSL_ERROR_BUSY;
+    case EAGAIN:
+    case EINTR:		return GSL_ERROR_TEMP;
+    case EINVAL:
+    case EFAULT:
+    case EBADF:         return GSL_ERROR_INTERNAL;
+    default:            return fallback;
     }
 }
 
@@ -1271,7 +1587,7 @@ gsl_init (const GslConfigValue values[],
     4,				/* wave_chunk_big_pad */
     512,			/* dcache_block_size */
     1024 * 1024,		/* dcache_cache_memory */
-    57,				/* midi_kammer_note */
+    69,				/* midi_kammer_note */
     440,			/* kammer_freq */
   };
 
@@ -1322,12 +1638,14 @@ gsl_init (const GslConfigValue values[],
   gsl_cond_init (&global_thread_cond);
   main_thread_tdata = create_tdata ();
   g_assert (main_thread_tdata != NULL);
-  _gsl_init_data_handles ();
+  main_thread = gsl_thread_self ();
+  global_thread_list = gsl_ring_prepend (global_thread_list, main_thread);
+  _gsl_init_signal ();
+  _gsl_init_fd_pool ();
   _gsl_init_data_caches ();
   _gsl_init_engine_utils ();
   _gsl_init_loader_gslwave ();
   _gsl_init_loader_wav ();
-#if 0 /* aRts: not yet */
   _gsl_init_loader_oggvorbis ();
-#endif
+  _gsl_init_loader_mad ();
 }
