@@ -1,5 +1,5 @@
 /* GSL - Generic Sound Layer
- * Copyright (C) 2001 Tim Janik and Stefan Westerfeld
+ * Copyright (C) 2001-2002 Tim Janik and Stefan Westerfeld
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -30,13 +30,13 @@
 static void	usage (void)	G_GNUC_NORETURN;
 
 
-static guint   shift_argc = 0;
-static gchar **shift_argv = NULL;
+static guint         shift_argc = 0;
+static const gchar **shift_argv = NULL;
 
-static gchar*
+static const gchar*
 shift (void)
 {
-  gchar *arg;
+  const gchar *arg;
   
   if (shift_argc > 1)
     {
@@ -49,10 +49,10 @@ shift (void)
     arg = NULL;
   return arg;
 }
-static gchar*
+static const gchar*
 pshift (void)
 {
-  gchar *arg = shift ();
+  const gchar *arg = shift ();
   
   return arg ? arg : "";
 }
@@ -61,13 +61,13 @@ int
 main (int   argc,
       char *argv[])
 {
-  gchar *arg;
+  const gchar *arg;
   
   /* iir filter parameters */
   enum { FILTER_GNUPLOT, FILTER_SCAN } filter_mode = FILTER_GNUPLOT;
   const gchar *filter_label = 0;
   gdouble *a, *b;
-  guint order;
+  guint order = 0;
   
   shift_argc = argc;
   shift_argv = argv;
@@ -84,7 +84,7 @@ main (int   argc,
   
   if (strcmp (arg, "wave-scan") == 0)
     {
-      gchar *file = pshift ();
+      const gchar *file = pshift ();
       
       while (file)
 	{
@@ -99,7 +99,7 @@ main (int   argc,
 	      g_print ("Loader \"%s\" found %u waves in \"%s\":\n", fi->loader->name, fi->n_waves, file);
 	      for (i = 0; i < fi->n_waves; i++)
 		g_print ("%u) %s\n", i + 1, fi->waves[i].name);
-	      gsl_wave_file_info_free (fi);
+	      gsl_wave_file_info_unref (fi);
 	    }
 	  else
 	    g_print ("Failed to scan \"%s\": %s\n", file, gsl_strerror (error));
@@ -110,7 +110,7 @@ main (int   argc,
     }
   else if (strcmp (arg, "file-test") == 0)
     {
-      gchar *file = pshift ();
+      const gchar *file = pshift ();
       
       g_print ("file test for \"%s\":\n", file);
       g_print ("  is readable   : %s\n", gsl_strerror (gsl_check_file (file, "r")));
@@ -452,10 +452,10 @@ main (int   argc,
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
+      gsl_filter_tscheb2_hp (order, f, st, e, a, b);
       g_print ("# Highpass Tschebyscheff Type2 order=%u freq=%f steepness=%f (%f, %f) epsilon(s^2)=%f norm=%f:\n",
 	       order, f, st, GSL_PI - f, (GSL_PI - f) * (1.+st), e,
 	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
-      gsl_filter_tscheb2_hp (order, f, st, e, a, b);
       filter_label = "T2H";
     }
   else if (strcmp (arg, "t2p") == 0)
@@ -472,10 +472,10 @@ main (int   argc,
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
+      gsl_filter_tscheb2_bp (order, f1, f2, st, e, a, b);
       g_print ("# Bandpass Tschebyscheff Type2 order=%u freq1=%f freq2=%f steepness=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, f1, f2, st, e,
 	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
-      gsl_filter_tscheb2_bp (order, f1, f2, st, e, a, b);
       filter_label = "T2P";
     }
   else if (strcmp (arg, "t2s") == 0)
@@ -492,10 +492,10 @@ main (int   argc,
       a = g_new (gdouble, order + 1);
       b = g_new (gdouble, order + 1);
       
+      gsl_filter_tscheb2_bs (order, f1, f2, st, e, a, b);
       g_print ("# Bandstop Tschebyscheff Type2 order=%u freq1=%f freq2=%f steepness=%f epsilon(s^2)=%f norm=%f:\n",
 	       order, f1, f2, st, e,
 	       gsl_poly_eval (order, a, 1) / gsl_poly_eval (order, b, 1));
-      gsl_filter_tscheb2_bs (order, f1, f2, st, e, a, b);
       filter_label = "T2S";
     }
   else if (strcmp (arg, "scan") == 0)
@@ -505,7 +505,7 @@ main (int   argc,
   else if (strcmp (arg, "fir") == 0)
     {
       unsigned int iorder = atoi (pshift ());
-      unsigned int n_points = 0, i = 0;
+      unsigned int n_points = 0;
       
       double *freq = g_newa (double, argc / 2 + 1);
       double *value = g_newa (double, argc / 2 + 1);
