@@ -32,7 +32,7 @@ enum {
   VERBOSITY_BLOCKS,
   VERBOSITY_DATA,
   VERBOSITY_PADDING,
-  VERBOSITY_CHECKS,
+  VERBOSITY_CHECKS
 };
 static guint verbosity = VERBOSITY_SETUP;
 
