@@ -474,13 +474,18 @@ void StdScheduleNode::rebuildConn()
 			ci != outConn[c]->autoDisconnect.end(); ci++)
 		{
 			AudioPort *dest = (*ci)->audioPort();
-			assert(dest);
 
-			gsl_trans_add(trans,
-				gsl_job_connect(outConn[c]->parent->gslModule,
+			if (dest) {
+				gsl_trans_add(trans,
+						gsl_job_connect(outConn[c]->parent->gslModule,
 								outConn[c]->gslEngineChannel,
 								dest->parent->gslModule,
 								dest->gslEngineChannel));
+			}
+			else
+			{
+				arts_debug( "no audio port: %s for %s", ( *ci )->name().c_str(), _object->_interfaceName().c_str() );
+			}
 		}
 	}
 	gsl_trans_commit(trans);
