@@ -1273,7 +1273,7 @@ EnumDef Object_stub::_queryEnum(const string& name)
 	_connection->qSendBuffer(request);
 
 	result = Dispatcher::the()->waitForResult(requestID,_connection);
-	if(!result) return EnumDef(); // error occured
+	if(!result) return EnumDef(); // error occurred
 	EnumDef _returnCode(*result);
 	delete result;
 	return _returnCode;
@@ -1333,7 +1333,7 @@ std::string Arts::Object_stub::_toString()
 	_connection->qSendBuffer(request);
 
 	result = Arts::Dispatcher::the()->waitForResult(requestID,_connection);
-	if(!result) return""; // error occured
+	if(!result) return""; // error occurred
 	std::string returnCode;
 	result->readString(returnCode);
 	delete result;
@@ -1351,7 +1351,7 @@ bool Arts::Object_stub::_isCompatibleWith(const std::string& interfacename)
 	_connection->qSendBuffer(request);
 
 	result = Arts::Dispatcher::the()->waitForResult(requestID,_connection);
-	if(!result) return false; // error occured
+	if(!result) return false; // error occurred
 	bool returnCode = result->readBool();
 	delete result;
 	return returnCode;
@@ -1408,7 +1408,7 @@ std::string Arts::Object_stub::_addChild(Arts::Object child, const std::string&
 	_connection->qSendBuffer(request);
 
 	result = Arts::Dispatcher::the()->waitForResult(requestID,_connection);
-	if(!result) return""; // error occured
+	if(!result) return""; // error occurred
 	std::string returnCode;
 	result->readString(returnCode);
 	delete result;
@@ -1426,7 +1426,7 @@ bool Arts::Object_stub::_removeChild(const std::string& name)
 	_connection->qSendBuffer(request);
 
 	result = Arts::Dispatcher::the()->waitForResult(requestID,_connection);
-	if(!result) return false; // error occured
+	if(!result) return false; // error occurred
 	bool returnCode = result->readBool();
 	delete result;
 	return returnCode;
@@ -1461,7 +1461,7 @@ std::vector<std::string> * Arts::Object_stub::_queryChildren()
 
 	result = Arts::Dispatcher::the()->waitForResult(requestID,_connection);
 	std::vector<std::string> *_returnCode = new std::vector<std::string>;
-	if(!result) return _returnCode; // error occured
+	if(!result) return _returnCode; // error occurred
 	result->readStringSeq(*_returnCode);
 	delete result;
 	return _returnCode;
