@@ -17,7 +17,7 @@
     the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
     Boston, MA 02111-1307, USA.
 */
-#include "core.h"
+#include "common.h"
 
 using namespace Arts;
 
