@@ -280,7 +280,7 @@ string createTypeCode(string type, const string& name, long model,
 			result = "stream.readFloatSeq("+name+")";
 		if(model==MODEL_RES_READ)
 		{
-			result = indent + "if(!result) return 0.0; // error occured\n";
+			result = indent + "if(!result) return 0.0; // error occurred\n";
 			result += indent + "float returnCode = result->readFloat();\n";
 			result += indent + "delete result;\n";
 			result += indent + "return returnCode;\n";
@@ -289,7 +289,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<float> *_returnCode ="
 												" new std::vector<float>;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "result->readFloatSeq(*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -330,7 +330,7 @@ string createTypeCode(string type, const string& name, long model,
 			result = "stream.readBoolSeq("+name+")";
 		if(model==MODEL_RES_READ)
 		{
-			result = indent + "if(!result) return false; // error occured\n";
+			result = indent + "if(!result) return false; // error occurred\n";
 			result += indent + "bool returnCode = result->readBool();\n";
 			result += indent + "delete result;\n";
 			result += indent + "return returnCode;\n";
@@ -339,7 +339,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<bool> *_returnCode ="
 												" new std::vector<bool>;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "result->readBoolSeq(*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -380,7 +380,7 @@ string createTypeCode(string type, const string& name, long model,
 			result = "stream.readByteSeq("+name+")";
 		if(model==MODEL_RES_READ)
 		{
-			result = indent + "if(!result) return 0; // error occured\n";
+			result = indent + "if(!result) return 0; // error occurred\n";
 			result += indent + "Arts::mcopbyte returnCode = result->readByte();\n";
 			result += indent + "delete result;\n";
 			result += indent + "return returnCode;\n";
@@ -389,7 +389,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<Arts::mcopbyte> *_returnCode ="
 										" new std::vector<Arts::mcopbyte>;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "result->readByteSeq(*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -433,7 +433,7 @@ string createTypeCode(string type, const string& name, long model,
 			result = "stream.readLongSeq("+name+")";
 		if(model==MODEL_RES_READ)
 		{
-			result = indent + "if(!result) return 0; // error occured\n";
+			result = indent + "if(!result) return 0; // error occurred\n";
 			result += indent + "long returnCode = result->readLong();\n";
 			result += indent + "delete result;\n";
 			result += indent + "return returnCode;\n";
@@ -442,7 +442,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<long> *_returnCode ="
 												" new std::vector<long>;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "result->readLongSeq(*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -489,7 +489,7 @@ string createTypeCode(string type, const string& name, long model,
 				   + indent + "request->readStringSeq("+name+");\n";
 		if(model==MODEL_RES_READ)
 		{
-			result = indent + "if(!result) return\"\"; // error occured\n";
+			result = indent + "if(!result) return\"\"; // error occurred\n";
 			result += indent + "std::string returnCode;\n";
 			result += indent + "result->readString(returnCode);\n";
 			result += indent + "delete result;\n";
@@ -499,7 +499,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<std::string> *_returnCode ="
 											" new std::vector<std::string>;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "result->readStringSeq(*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -567,7 +567,7 @@ string createTypeCode(string type, const string& name, long model,
 		if(model==MODEL_RES_READ)
 		{
 			result = indent +
-					"if(!result) return "+type+"(); // error occured\n";
+					"if(!result) return "+type+"(); // error occurred\n";
 			result += indent+ type + " _returnCode(*result);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -576,7 +576,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<"+type+"> *_returnCode ="
 											" new std::vector<"+type+">;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "Arts::readTypeSeq(*result,*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -598,7 +598,7 @@ string createTypeCode(string type, const string& name, long model,
 		if(model==MODEL_RES_READ)
 		{
 			result = indent  +
-						"if(!result) return ("+type+")0; // error occured\n";
+						"if(!result) return ("+type+")0; // error occurred\n";
 			result += indent + type+" returnCode = ("+
 												type+")result->readLong();\n";
 			result += indent + "delete result;\n";
@@ -631,7 +631,7 @@ string createTypeCode(string type, const string& name, long model,
 			result = "Arts::readObjectSeq(stream,"+name+")";
 		if(model==MODEL_RES_READ)
 		{
- 			result = indent + "if (!result) return "+type+"::null();\n"; // error occured\n";
+ 			result = indent + "if (!result) return "+type+"::null();\n"; // error occurred\n";
 			result += indent + type+"_base* returnCode;\n";
 			result += indent + "Arts::readObject(*result,returnCode);\n";
 			result += indent + "delete result;\n";
@@ -641,7 +641,7 @@ string createTypeCode(string type, const string& name, long model,
 		{
 			result = indent + "std::vector<"+type+"> *_returnCode ="
 												" new std::vector<"+type+">;\n";
-			result += indent + "if(!result) return _returnCode; // error occured\n";
+			result += indent + "if(!result) return _returnCode; // error occurred\n";
 			result += indent + "Arts::readObjectSeq(*result,*_returnCode);\n";
 			result += indent + "delete result;\n";
 			result += indent + "return _returnCode;\n";
@@ -677,7 +677,7 @@ string createTypeCode(string type, const string& name, long model,
 	}
 	else
 	{
-		fprintf(stderr,"error: undefined type %s occured\n",type.c_str());
+		fprintf(stderr,"error: undefined type %s occurred\n",type.c_str());
 		exit(1);
 	}
 
