@@ -55,11 +55,10 @@ public:
 	{
 		pfd = fileno(pfile);
 
-		int rc = fcntl(pfd, F_SETFL, O_NONBLOCK);
-		assert(rc != -1);
+		fcntl(pfd, F_SETFL, O_NONBLOCK);
 
 		/*
-		 * calculate stream buffer paramters
+		 * calculate stream buffer parameters
 		 */
 		float streamBufferTime;
 		packets = 7;
