@@ -291,7 +291,7 @@ int main(int argc, char **argv)
 	SoundServerV2 server;
 	AudioManager audioManager;
 
-	if (fabsf(cfgVolume) > 1e-10)
+	if (fabs(cfgVolume) > 1e-10)
 		server.outVolume().scaleFactor(cfgVolume);
 
 	if (cfgAutoSuspend)
