@@ -503,7 +503,7 @@ int executeCommand(Arts::SoundServerV2 server, int argc, char **argv)
 		return 0;
 	}
 	if(!strcmp(argv[0], "volume") && (argc == 1)) {
-		cout << getVolume(server) << endl;;
+		cout << getVolume(server) << endl;
 		return 0;
 	}
 
