@@ -38,8 +38,9 @@ namespace Arts {
 		RealtimeStatus realtimeStatus();
 		long secondsUntilSuspend();
 		bool suspend();
+		bool suspended();
 		bool terminate();
 	};
-};
+}
 
 #endif /* VERBOSESOUNDSERVER_IMPL_H */
